//user:    GSC
//date:    18/11/2024
//ver:     1.0.0
//class:   dcs_dashboard_set
//note:    dcs_dashboard_set event trigger
	

public class dcsDashboardSetEventTrigger extends com.geowebframework.dataservice.querybuilder.EventTrigger {
	
	final String CLASS_NAME = "dcs_dashboard_set";
	final String CLASS_TABLE_NAME = "dcs_dashboard_set";
	
	
	public boolean beforeInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeInsert()");
		
		String radix = "DSH_BRD_SET_";
		String cod_column = "cod_dashboard_set";
		Integer leadingZeros = 6;
		String leadedInt = String.format("%0"+leadingZeros+"d", 1);
		String query = "SELECT "+cod_column+" FROM "+CLASS_TABLE_NAME+" ORDER BY "+cod_column+" asc";
		def res = services.queryService.executeQuery(query, null);
		if(res.size()!=0){
			String cod_model_last = res[res.size()-1][cod_column];
			Integer lastInt = Integer.parseInt(cod_model_last.substring(cod_model_last.indexOf(radix)+radix.length()));
			lastInt++;
			leadedInt = String.format("%0"+leadingZeros+"d", lastInt);
		}
		String newCod = radix+leadedInt;
		valuesMap[cod_column] = newCod;
		
		//check if this.gwa func code had already been taken
		String cod_function = valuesMap.cod_function;
		if(cod_function==null) throw new RuntimeException("Missing required paramter 'cod_function'");
		query = "SELECT cod_function FROM "+CLASS_TABLE_NAME;
		res = services.queryService.executeQuery(query, null);
		if(res.size()!=0){
			Boolean found = false;
			res.each { r ->
				if(r.cod_function==cod_function) found = true;
			};
			if(found) throw new RuntimeException("This cod_function: '"+cod_function+"', is already used by another Dashboard Set");
		}
		
		
		return true;
	};
	
	public boolean afterInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterInsert()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};
	
	public boolean beforeUpdate(HashMap<String,Object> valuesMap, HashMap<String,Object> oldvaluesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeUpdate()");
		
		
		//check if this.gwa func code had already been taken
		String cod_dashboard_set = oldvaluesMap.cod_dashboard_set;
		String cod_function = valuesMap.cod_function;
		if(cod_function!=null){
			String query = "SELECT cod_function FROM "+CLASS_TABLE_NAME;
			def res = services.queryService.executeQuery(query, null);
			if(res.size()!=0){
				Boolean found = false;
				res.each { r ->
					if(r.cod_function==cod_function && r.cod_dashboard_set!=cod_dashboard_set) found = true;
				};
				if(found) throw new RuntimeException("This cod_function: '"+cod_function+"', is already used by another Dashboard Set");
			}
		}
		
		return true;
	};
	
	public boolean afterUpdate(HashMap<String,Object> valuesMap,HashMap<String,Object> oldvaluesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterUpdate()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};
	
	public boolean beforeDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeDelete()");
		return true;
	};
	
	public boolean afterDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterDelete()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};

} 