//user:    GSC
//date:    18/11/2024
//ver:     1.0.0
//class:   dcs_r_dashboard_set_dashboard
//note:    dcs_r_dashboard_set_dashboard event trigger

//related to issue #1428 geowebframework
public class dcsRDashboardSetDashboardSetEventTrigger extends com.geowebframework.dataservice.querybuilder.EventTrigger {
	
	final String CLASS_NAME = "dcs_r_dashboard_set_dashboard";
	final String CLASS_TABLE_NAME = "dcs_r_dashboard_set_dashboard";
	
	public boolean beforeInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeInsert()");
		
		
		Integer is_default = valuesMap.is_default;
		
		def cod_dashboard_set = valuesMap.cod_dashboard_set;
		
		def cod_dashboard = valuesMap.cod_dashboard;
		
		if(valuesMap.is_hidden==null) valuesMap.is_hidden = 0;
		
		//handle default and order
		def query = "SELECT * FROM "+CLASS_TABLE_NAME+" WHERE cod_dashboard_set=#{map.cod_dashboard_set}";
		def dashboardList = services.queryService.executeQuery(query, [cod_dashboard_set: cod_dashboard_set, cod_dashboard: cod_dashboard]);
		if(dashboardList.size()==0){
			//the first insert is always set as default, by default
			if(is_default==null || is_default!=1){
				valuesMap.is_default = 1;
			}
			valuesMap.dashboard_order = 1;
		}else{
			if(is_default==null) valuesMap.is_default = 0;
			valuesMap.dashboard_order = dashboardList.size()+1;
		}
		
		return true;
	};
	
	public boolean afterInsert(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterInsert()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};
	
	public boolean beforeUpdate(HashMap<String,Object> valuesMap, HashMap<String,Object> oldvaluesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeUpdate()");
		
		def is_default = valuesMap.is_default;
		if(is_default==null) is_default = oldvaluesMap.is_default;
		def is_hidden = valuesMap.is_hidden;
		
		def cod_dashboard_set = valuesMap.cod_dashboard_set;
		if(cod_dashboard_set==null) cod_dashboard_set = oldvaluesMap.cod_dashboard_set;
		
		def cod_dashboard = valuesMap.cod_dashboard;
		if(cod_dashboard==null) cod_dashboard = oldvaluesMap.cod_dashboard;
		
		if(is_default==1){
			//set is_default=0 to all others dashboarda
			String query = "UPDATE "+CLASS_TABLE_NAME+" SET is_default=#{map.is_default} WHERE cod_dashboard_set=#{map.cod_dashboard_set} AND cod_dashboard!=#{map.cod_dashboard}";
			services.queryService.executeUpdateQuery(query, [is_default: 0, cod_dashboard_set: cod_dashboard_set, cod_dashboard: cod_dashboard]);
			
			//set dashboard_order=1 to this dashboard
			query = "UPDATE "+CLASS_TABLE_NAME+" SET dashboard_order=1 WHERE cod_dashboard_set=#{map.cod_dashboard_set} AND cod_dashboard=#{map.cod_dashboard}";
			services.queryService.executeUpdateQuery(query, [cod_dashboard_set: cod_dashboard_set, cod_dashboard: cod_dashboard]);
			
			//select all others ordered
			query = "SELECT * FROM "+CLASS_TABLE_NAME+" WHERE cod_dashboard_set=#{map.cod_dashboard_set} AND cod_dashboard!=#{map.cod_dashboard} ORDER BY dashboard_order";
			def dashboardList = services.queryService.executeQuery(query, [cod_dashboard_set: cod_dashboard_set, cod_dashboard: cod_dashboard]);
			//and compute new order
			def order = 2;
			for(def d in dashboardList){
				query = "UPDATE "+CLASS_TABLE_NAME+" SET dashboard_order=#{map.order} WHERE cod_dashboard_set=#{map.cod_dashboard_set} AND cod_dashboard=#{map.cod_dashboard}";
				services.queryService.executeUpdateQuery(query, [order: order, cod_dashboard_set: d.cod_dashboard_set, cod_dashboard: d.cod_dashboard]);
				order++;
			}
			
			//default dashboard cannot to be hidden too
			valuesMap.is_hidden = 0;
		}else{
			if(is_default==0 && oldvaluesMap.is_default==1){
				throw new RuntimeException("Deve esserci sempre almeno una Dashboard impostata come principale");
			}
		}
		
		if(is_hidden==1 && is_default==1){
			throw new RuntimeException("La Dashboard principale non può essere nascosta");
		}
		
		return true;
	};
	
	public boolean afterUpdate(HashMap<String,Object> valuesMap,HashMap<String,Object> oldvaluesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterUpdate()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};
	
	public boolean beforeDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - beforeDelete()");
		return true;
	};
	
	public boolean afterDelete(HashMap<String,Object> valuesMap){
		log.debug("gwClassName: "+CLASS_NAME+" - afterDelete()");
		
		services.gwMetadataCacheService.clearAllStartingWith("gwDashboardSetService_");
		
		return true;
	};

} 